@@ -1,9 +1,9 @@
-/* $Id: abs_op.hpp 2625 2012-12-23 14:34:12Z bradbell $ */
+/* $Id: abs_op.hpp 3320 2014-09-11 23:06:21Z bradbell $ */
 # ifndef CPPAD_ABS_OP_INCLUDED
 # define CPPAD_ABS_OP_INCLUDED
 
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-12 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-14 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the 
@@ -14,10 +14,8 @@ Please visit http://www.coin-or.org/CppAD/ for information on other licenses.
 -------------------------------------------------------------------------- */
 
 
-CPPAD_BEGIN_NAMESPACE
+namespace CppAD { // BEGIN_CPPAD_NAMESPACE
 /*!
-\defgroup abs_op_hpp abs_op.hpp
-\{
 \file abs_op.hpp
 Forward and reverse mode calculations for z = abs(x).
 */
@@ -34,23 +32,60 @@ The C++ source code corresponding to this operation is
 */
 template <class Base>
 inline void forward_abs_op(
-	size_t j           ,
+	size_t p           ,
+	size_t q           ,
 	size_t i_z         ,
 	size_t i_x         ,
-	size_t nc_taylor   , 
+	size_t cap_order   , 
 	Base*  taylor      )
 {
 	// check assumptions
 	CPPAD_ASSERT_UNKNOWN( NumArg(AbsOp) == 1 );
 	CPPAD_ASSERT_UNKNOWN( NumRes(AbsOp) == 1 );
-	CPPAD_ASSERT_UNKNOWN( i_x < i_z );
-	CPPAD_ASSERT_UNKNOWN( j < nc_taylor );
+	CPPAD_ASSERT_UNKNOWN( q < cap_order );
+	CPPAD_ASSERT_UNKNOWN( p <= q );
 
 	// Taylor coefficients corresponding to argument and result
-	Base* x = taylor + i_x * nc_taylor;
-	Base* z = taylor + i_z * nc_taylor;
+	Base* x = taylor + i_x * cap_order;
+	Base* z = taylor + i_z * cap_order;
 
-	z[j] = sign(x[0]) * x[j];
+	for(size_t j = p; j <= q; j++)
+		z[j] = sign(x[0]) * x[j];
+}
+
+/*!
+Multiple directions forward mode Taylor coefficient for op = AbsOp.
+
+The C++ source code corresponding to this operation is
+\verbatim
+	z = abs(x)
+\endverbatim
+
+\copydetails forward_unary1_op_dir
+*/
+template <class Base>
+inline void forward_abs_op_dir(
+	size_t q           ,
+	size_t r           ,
+	size_t i_z         ,
+	size_t i_x         ,
+	size_t cap_order   , 
+	Base*  taylor      )
+{
+	// check assumptions
+	CPPAD_ASSERT_UNKNOWN( NumArg(AbsOp) == 1 );
+	CPPAD_ASSERT_UNKNOWN( NumRes(AbsOp) == 1 );
+	CPPAD_ASSERT_UNKNOWN( 0 < q );
+	CPPAD_ASSERT_UNKNOWN( q < cap_order );
+
+	// Taylor coefficients corresponding to argument and result
+	size_t num_taylor_per_var = (cap_order-1) * r + 1;
+	Base* x = taylor + i_x * num_taylor_per_var;
+	Base* z = taylor + i_z * num_taylor_per_var;
+
+	size_t m = (q-1) * r + 1;
+	for(size_t ell = 0; ell < r; ell++)
+		z[m + ell] = sign(x[0]) * x[m + ell];
 }
 
 /*!
@@ -67,19 +102,18 @@ template <class Base>
 inline void forward_abs_op_0(
 	size_t i_z         ,
 	size_t i_x         ,
-	size_t nc_taylor   , 
+	size_t cap_order   , 
 	Base*  taylor      )
 {
 
 	// check assumptions
 	CPPAD_ASSERT_UNKNOWN( NumArg(AbsOp) == 1 );
 	CPPAD_ASSERT_UNKNOWN( NumRes(AbsOp) == 1 );
-	CPPAD_ASSERT_UNKNOWN( i_x < i_z );
-	CPPAD_ASSERT_UNKNOWN( 0 < nc_taylor );
+	CPPAD_ASSERT_UNKNOWN( 0 < cap_order );
 
 	// Taylor coefficients corresponding to argument and result
-	Base x0 = *(taylor + i_x * nc_taylor);
-	Base* z = taylor + i_z * nc_taylor;
+	Base x0 = *(taylor + i_x * cap_order);
+	Base* z = taylor + i_z * cap_order;
 
 	z[0] = abs(x0);
 }
@@ -99,7 +133,7 @@ inline void reverse_abs_op(
 	size_t      d            ,
 	size_t      i_z          ,
 	size_t      i_x          ,
-	size_t      nc_taylor    , 
+	size_t      cap_order    , 
 	const Base* taylor       ,
 	size_t      nc_partial   ,
 	Base*       partial      )
@@ -108,12 +142,11 @@ inline void reverse_abs_op(
 	// check assumptions
 	CPPAD_ASSERT_UNKNOWN( NumArg(AbsOp) == 1 );
 	CPPAD_ASSERT_UNKNOWN( NumRes(AbsOp) == 1 );
-	CPPAD_ASSERT_UNKNOWN( i_x < i_z );
-	CPPAD_ASSERT_UNKNOWN( d < nc_taylor );
+	CPPAD_ASSERT_UNKNOWN( d < cap_order );
 	CPPAD_ASSERT_UNKNOWN( d < nc_partial );
 
 	// Taylor coefficients and partials corresponding to argument
-	const Base* x  = taylor  + i_x * nc_taylor;
+	const Base* x  = taylor  + i_x * cap_order;
 	Base* px       = partial + i_x * nc_partial;
 
 	// Taylor coefficients and partials corresponding to result
@@ -123,6 +156,5 @@ inline void reverse_abs_op(
 		px[j] += sign(x[0]) * pz[j];
 }
 
-/*! \} */
-CPPAD_END_NAMESPACE
+} // END_CPPAD_NAMESPACE
 # endif
