@@ -1,9 +1,9 @@
-// $Id: sparse_set.hpp 2625 2012-12-23 14:34:12Z bradbell $
+// $Id: sparse_set.hpp 3301 2014-05-24 05:20:21Z bradbell $
 # ifndef CPPAD_SPARSE_SET_INCLUDED
 # define CPPAD_SPARSE_SET_INCLUDED
 
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-12 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-14 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the 
@@ -18,10 +18,8 @@ Please visit http://www.coin-or.org/CppAD/ for information on other licenses.
 # include <cppad/local/cppad_assert.hpp>
 
 
-CPPAD_BEGIN_NAMESPACE
+namespace CppAD { // BEGIN_CPPAD_NAMESPACE
 /*!
-\defgroup sparse_set_hpp sparse_set.hpp
-\{
 \file sparse_set.hpp
 Vector of sets of positive integers stored as std::set<size_t>.
 */
@@ -371,6 +369,5 @@ void sparsity_user2internal(
 	return;
 }
 
-/*! \} */
-CPPAD_END_NAMESPACE
+} // END_CPPAD_NAMESPACE
 # endif
