@@ -1,6 +1,6 @@
-/* $Id: mul_level.cpp 2506 2012-10-24 19:36:49Z bradbell $ */
+/* $Id: mul_level.cpp 3071 2014-01-01 04:02:27Z bradbell $ */
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-12 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-13 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the 
@@ -35,13 +35,10 @@ bool One(void)
 	typedef CppAD::AD<ADdouble> ADDdouble;   // for two levels of taping
 	size_t n = 2;                            // dimension for example
 
-	CPPAD_TESTVECTOR(ADdouble)   a_x(n);
-	CPPAD_TESTVECTOR(ADDdouble) aa_x(n);
-
 	// value of the independent variables
-	a_x[0] = 2.; a_x[1] = 3.;
-	Independent(a_x);
-	aa_x[0] = a_x[0]; aa_x[1] = a_x[1];
+	CPPAD_TESTVECTOR(ADDdouble) aa_x(n);
+	aa_x[0] = 1.; aa_x[1] = 3.; // test conversion double to AD< AD<double> >
+	aa_x[0] = 2. * aa_x[0];     // test double * AD< AD<double> >
 	CppAD::Independent(aa_x);
 
 	// compute the function f(x) = 2 * x[0] * x[1]
@@ -49,7 +46,12 @@ bool One(void)
 	aa_f[0] = 2. * aa_x[0] * aa_x[1];
 	CppAD::ADFun<ADdouble> F(aa_x, aa_f);
 
-	// re-evaluate f(2, 3) (must get proper deepedence on a_x).
+	// value of the independent variables
+	CPPAD_TESTVECTOR(ADdouble)   a_x(n);
+	a_x[0] = 2.; a_x[1] = 3.;
+	Independent(a_x);
+
+	// re-evaluate f(2, 3) (must get deepedence on a_x).
 	size_t p = 0;
 	CPPAD_TESTVECTOR(ADdouble) a_fp(1);
 	a_fp    = F.Forward(p, a_x);
@@ -200,7 +202,7 @@ bool adolc(void)
 
 	// compute the d/dx of f'(x) * v = f''(x) * v
 	size_t m      = n;                     // # dependent in f'(x)
-	double *v = 0, *ddf_v = 0;
+	double *v = CPPAD_NULL, *ddf_v = CPPAD_NULL;
 	v     = CPPAD_TRACK_NEW_VEC(m, v);     // track v = new double[m]
 	ddf_v = CPPAD_TRACK_NEW_VEC(n, ddf_v); // track ddf_v = new double[n]
 	for(j = 0; j < n; j++)
