@@ -1,6 +1,6 @@
-/* $Id: optimize.cpp 2506 2012-10-24 19:36:49Z bradbell $ */
+/* $Id: optimize.cpp 3634 2015-02-09 14:09:03Z bradbell $ */
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-12 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-13 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the 
@@ -9,11 +9,300 @@ the terms of the
 A copy of this license is included in the COPYING file of this distribution.
 Please visit http://www.coin-or.org/CppAD/ for information on other licenses.
 -------------------------------------------------------------------------- */
+// 2DO: Test that optimize.hpp use of base_atomic<Base>::rev_sparse_jac works.
 
 # include <limits>
 # include <cppad/cppad.hpp>
 
+
 namespace {
+	// note this enum type is not part of the API (but its values are)
+	CppAD::atomic_base<double>::option_enum atomic_sparsity_option;
+	//
+	// ----------------------------------------------------------------
+	// Test nested conditional expressions.
+	int nested_cond_exp(void)
+	{	bool ok = true;
+		using CppAD::AD;
+		using CppAD::vector;
+	
+		// independent variable vector 
+		vector< AD<double> > ax(2), ay(1);
+		ax[0] = 1.0;
+		ax[1] = 2.0;
+		Independent(ax);
+	
+		// first conditional expression
+		AD<double> ac1 = CondExpLe(ax[0], ax[1], 2.0 * ax[0], 3.0 * ax[1] );
+	
+		// second conditional expression
+		AD<double> ac2 = CondExpGe(ax[0], ax[1], 4.0 * ax[0], 5.0 * ax[1] );
+	
+		// third conditional expression
+		AD<double> ac3 = CondExpLt(ax[0], ax[1], 6.0 * ac1, 7.0 * ac2 );
+	
+		// create function object f : ax -> ay
+		ay[0] = ac3;
+		CppAD::ADFun<double> f(ax, ay);
+	
+		// now optimize the operation sequence
+		f.optimize();
+	
+		// now zero order forward
+		vector<double> x(2), y(1);
+		for(size_t i = 0; i < 3; i++)
+		{	x[0] = 1.0 - double(i);
+			x[1] = - x[0];
+			y    = f.Forward(0, x);
+			//
+			// first conditional expression
+			double c1;
+			if( x[0] <= x[1] )
+				c1 = 2.0 * x[0];
+			else	c1 = 3.0 * x[1];
+			//
+			// second conditional expression
+			double c2;
+			if( x[0] >= x[1] )
+				c2 = 4.0 * x[0];
+			else	c2 = 5.0 * x[1];
+	
+			// third conditional expression
+			double c3;
+			if( x[0] < x[1] )
+				c3 = 6.0 * c1;
+			else	c3 = 7.0 * c2;
+	
+			ok &= y[0] == c3;
+		}
+		return ok;
+	}
+	// ----------------------------------------------------------------
+	// Test for bug where checkpoint function did not depend on
+	// the operands in the logical comparison because of the CondExp
+	// sparsity pattern.
+	void j_algo( 
+		const CppAD::vector< CppAD::AD<double> >& ax ,
+		      CppAD::vector< CppAD::AD<double> >& ay )
+	{	ay[0] = CondExpGt(ax[0], ax[1], ax[2], ax[3]); }
+	
+	bool atomic_cond_exp_sparsity(void)
+	{	bool ok = true;
+		using CppAD::AD;
+		using CppAD::vector;
+	
+		// Create a checkpoint version of the function g
+		vector< AD<double> > au(4), av(1);
+		for(size_t i = 0; i < 4; i++)
+			au[i] = AD<double>(i);
+		CppAD::checkpoint<double> j_check("j_check", j_algo, au, av);
+	
+		// independent variable vector 
+		vector< AD<double> > ax(2), ay(1);
+		ax[0] = 1.;
+		ax[1] = 1.;
+		Independent(ax);
+	
+		// call atomic function that does not get used
+		for(size_t i = 0; i < 4; i++) 
+			au[i] = ax[0] + AD<double>(i + 1) * ax[1];
+		j_check(au, ay);
+	
+		// create function object f : ax -> ay
+		CppAD::ADFun<double> f(ax, ay);
+
+	
+		// now optimize the operation sequence
+		j_check.option( atomic_sparsity_option );
+		f.optimize();
+	
+		// check result where true case is used; i.e., au[0] > au[1]
+		vector<double> x(2), y(1);
+		x[0] = 1.;
+		x[1] = -1;
+		y    = f.Forward(0, x);
+		ok  &= y[0] == x[0] + double(3) * x[1];
+		
+	
+		// check result where false case is used; i.e., au[0] <= au[1]
+		x[0] = 1.;
+		x[1] = 1;
+		y    = f.Forward(0, x);
+		ok  &= y[0] == x[0] + double(4) * x[1];
+		
+		return ok;
+	}
+	// -------------------------------------------------------------------
+	// Test conditional optimizing out call to an atomic function call
+	void k_algo(
+		const CppAD::vector< CppAD::AD<double> >& x ,
+		      CppAD::vector< CppAD::AD<double> >& y )
+	{	y[0] = x[0] + x[1]; }
+
+	void h_algo(
+		const CppAD::vector< CppAD::AD<double> >& x ,
+		      CppAD::vector< CppAD::AD<double> >& y )
+	{	y[0] = x[0] - x[1]; }
+
+	bool atomic_cond_exp(void)
+	{	bool ok = true;
+		typedef CppAD::vector< CppAD::AD<double> > ADVector;
+
+		// Create a checkpoint version of the function g
+		ADVector ax(2), ag(1), ah(1), ay(1);
+		ax[0] = 0.;
+		ax[1] = 1.;
+		CppAD::checkpoint<double> k_check("k_check", k_algo, ax, ag);
+		CppAD::checkpoint<double> h_check("h_check", h_algo, ax, ah);
+
+		// independent variable vector 
+		Independent(ax);
+
+		// atomic function calls that get conditionally used
+		k_check(ax, ag);
+		h_check(ax, ah);
+
+		// conditional expression
+		ay[0] = CondExpLt(ax[0], ax[1], ag[0], ah[0]); 
+	
+		// create function object f : ax -> ay
+		CppAD::ADFun<double> f;
+		f.Dependent(ax, ay);
+	
+		// use zero order to evaluate when condition is true
+		CppAD::vector<double>  x(2), dx(2);
+		CppAD::vector<double>  y(1), dy(1), w(1);
+		x[0] = 3.;
+		x[1] = 4.;
+		y    = f.Forward(0, x);
+		ok  &= y[0] == x[0] + x[1];
+
+		// before optimize
+		k_check.option( atomic_sparsity_option );
+		h_check.option( atomic_sparsity_option );
+		ok  &= f.number_skip() == 0;
+
+		// now optimize the operation sequence
+		f.optimize();
+
+		// optimized zero order forward when condition is false
+		x[0] = 4.;
+		x[1] = 3.;
+		y    = f.Forward(0, x);
+		ok  &= y[0] == x[0] - x[1];
+
+		// after optimize can skip either call to g or call to h
+		ok  &= f.number_skip() == 1;
+
+		// optimized first order forward
+		dx[0] = 2.;
+		dx[1] = 1.;
+		dy    = f.Forward(1, dx);
+		ok   &= dy[0] == dx[0] - dx[1];
+
+		// optimized first order reverse
+		w[0]  = 1.;
+		dx    = f.Reverse(1, w);
+		ok   &= dx[0] == 1.;
+		ok   &= dx[1] == -1.;
+	
+		return ok;
+	}
+	// -------------------------------------------------------------------
+	// Test of optimizing out arguments to an atomic function
+	void g_algo( 
+		const CppAD::vector< CppAD::AD<double> >& ax ,
+		      CppAD::vector< CppAD::AD<double> >& ay )
+	{	ay = ax; }
+
+	bool atomic_no_used(void)
+	{	bool ok = true;
+		using CppAD::AD;
+		using CppAD::vector;
+	
+		// Create a checkpoint version of the function g
+		vector< AD<double> > ax(2), ay(2), az(1);
+		ax[0] = 0.;
+		ax[1] = 1.;
+		CppAD::checkpoint<double> g_check("g_check", g_algo, ax, ay);
+	
+		// independent variable vector 
+		Independent(ax);
+	
+		// call atomic function that does not get used
+		g_check(ax, ay);
+	
+		// conditional expression
+		az[0] = CondExpLt(ax[0], ax[1], ax[0] + ax[1], ax[0] - ax[1]); 
+		
+		// create function object f : ax -> az
+		CppAD::ADFun<double> f(ax, az);
+
+		// number of variables before optimization
+		// (include ay[0] and ay[1])
+		size_t n_before = f.size_var();
+		
+		// now optimize the operation sequence
+		g_check.option( atomic_sparsity_option );
+		f.optimize();
+
+		// number of variables after optimization 
+		// (does not include ay[0] and ay[1])
+		size_t n_after = f.size_var();
+		ok            &= n_after + 2 == n_before;
+	
+		// check optimization works ok
+		vector<double> x(2), z(1);
+		x[0] = 4.;
+		x[1] = 3.;
+		z    = f.Forward(0, x);
+		ok  &= z[0] == x[0] - x[1];
+		
+		return ok;
+	}
+	bool atomic_arguments(void)
+	{	bool ok = true;
+		using CppAD::AD;
+		using CppAD::vector;
+		vector< AD<double> > au(2), aw(2), ax(2), ay(1);
+
+		// create atomic function corresponding to g_algo
+		au[0] = 1.0;
+		au[1] = 2.0;
+		CppAD::checkpoint<double> g_check("g_algo", g_algo, au, ax);
+
+		// start recording a new function
+		CppAD::Independent(ax);
+
+		// now use g_check during the recording
+		au[0] = ax[0] + ax[1]; // this argument requires a new variable
+		au[1] = ax[0] - ax[1]; // this argument also requires a new variable
+		g_check(au, aw);
+
+		// now create f(x) = x_0 - x_1 
+		ay[0] = aw[0];
+		CppAD::ADFun<double> f(ax, ay);
+
+		// number of variables before optimization
+		size_t n_before = f.size_var();
+ 
+		// now optimize f so that the calculation of au[1] is removed
+		g_check.option( atomic_sparsity_option );
+		f.optimize();
+
+		// check difference in number of variables
+		size_t n_after = f.size_var();
+		ok &= n_before == n_after + 1;
+
+		// now compute and check a forward mode calculation
+		vector<double> x(2), y(1);
+		x[0] = 5.0;
+		x[1] = 6.0;
+		y    = f.Forward(0, x);
+		ok  &= (y[0] == x[0] + x[1]); 
+
+		return ok;
+	}
 
 	// -------------------------------------------------------------------
 	// Test the reverse dependency analysis optimization
@@ -112,7 +401,7 @@ namespace {
 		y = F.Forward(0, x);
 		depend_fun(x, check, original, opt);
 		for(i = 0; i < m; i++)
-			ok &= (y[i] == check[i]);
+			ok &= CppAD::NearEqual(y[i], check[i], 1e-10, 1e-10);
 	
 		// Check size before optimization
 		ok &= F.size_var() == (n + 1 + original);
@@ -127,7 +416,7 @@ namespace {
 		// (should have already been checked if NDEBUG not defined)
 		y = F.Forward(0, x);
 		for(i = 0; i < m; i++)
-			ok &= (y[i] == check[i]);
+			ok &= CppAD::NearEqual(y[i], check[i], 1e-10, 1e-10);
 	
 		return ok;
 	}
@@ -361,7 +650,7 @@ namespace {
 		y = F.Forward(0, x);
 		duplicate_fun(x, check, original, opt);
 		for(i = 0; i < m; i++)
-			ok &= (y[i] == check[i]);
+			ok &= CppAD::NearEqual(y[i], check[i], 1e-10, 1e-10);
 	
 		// Check size before optimization
 		ok &= F.size_var() == (n + 1 + original);
@@ -376,7 +665,7 @@ namespace {
 		// (should have already been checked if NDEBUG not defined)
 		y = F.Forward(0, x);
 		for(i = 0; i < m; i++)
-			ok &= (y[i] == check[i]);
+			ok &= CppAD::NearEqual(y[i], check[i], 1e-10, 1e-10);
 	
 		return ok;
 	}
@@ -713,7 +1002,7 @@ namespace {
 	
 		return ok;
 	}
-	bool forward_sparse_jacobian_csum()
+	bool forward_sparse_jacobian()
 	{	bool ok = true;
 		using namespace CppAD;
 	
@@ -721,12 +1010,13 @@ namespace {
 		size_t n = 3; 
 	
 		// dimension of the range space
-		size_t m = 2;
+		size_t m = 3;
 	
 		// independent variable vector 
 		CppAD::vector< AD<double> > X(n);
 		X[0] = 2.; 
 		X[1] = 3.;
+		X[2] = 4.;
 		Independent(X);
 	
 		// dependent variable vector
@@ -751,6 +1041,17 @@ namespace {
 		Check[index * n + 1] = true;
 		Check[index * n + 2] = true;
 		index++;
+
+		// Y[2] 
+		// 2DO: There is a subtitle issue that has to do with using reverse
+		// jacobian sparsity patterns during the optimization process.
+		// We need an option to include X[0] in the sparsity pattern
+		// so the optimizer can know it affects the results.
+		Y[index]             = CondExpLe(X[0], X[1], X[1]+X[1], X[2]-X[2]);
+		Check[index * n + 0] = false;
+		Check[index * n + 1] = true;
+		Check[index * n + 2] = true;
+		index++;
 	
 		// check final index
 		assert( index == m );
@@ -783,7 +1084,7 @@ namespace {
 	
 		return ok;
 	}
-	bool reverse_sparse_jacobian_csum()
+	bool reverse_sparse_jacobian()
 	{	bool ok = true;
 		using namespace CppAD;
 	
@@ -791,12 +1092,13 @@ namespace {
 		size_t n = 3; 
 	
 		// dimension of the range space
-		size_t m = 2;
+		size_t m = 3;
 	
 		// independent variable vector 
 		CppAD::vector< AD<double> > X(n);
 		X[0] = 2.; 
 		X[1] = 3.;
+		X[2] = 4.;
 		Independent(X);
 	
 		// dependent variable vector
@@ -821,6 +1123,13 @@ namespace {
 		Check[index * n + 1] = true;
 		Check[index * n + 2] = true;
 		index++;
+
+		// Y[2] 
+		Y[index]             = CondExpLe(X[0], X[1], X[1]+X[1], X[2]-X[2]);
+		Check[index * n + 0] = false;
+		Check[index * n + 1] = true;
+		Check[index * n + 2] = true;
+		index++;
 	
 		// check final index
 		assert( index == m );
@@ -850,26 +1159,37 @@ namespace {
 	
 		return ok;
 	}
-	bool reverse_sparse_hessian_csum(void)
+	bool reverse_sparse_hessian(void)
 	{	bool ok = true;
 		using CppAD::AD;
 		size_t i, j;
 	
-		size_t n = 2;
+		size_t n = 3;
 		CppAD::vector< AD<double> > X(n); 
 		X[0] = 1.;
 		X[1] = 2.;
+		X[2] = 3.;
 		CppAD::Independent(X);
 	
 		size_t m = 1;
 		CppAD::vector< AD<double> > Y(m);
-		Y[0] = (2. + X[0] + X[1] + 3.) * X[0];
+		Y[0] = CondExpGe( X[0], X[1], 
+			X[0] + (2. + X[1] + 3.) * X[1], 
+			X[0] + (2. + X[2] + 3.) * X[1]
+		);
 	
 		CppAD::vector<bool> check(n * n);
-		check[0 * n + 0] = true;  // partial w.r.t. x[0], x[0]
-		check[0 * n + 1] = true;  //                x[0], x[1]
-		check[1 * n + 0] = true;  //                x[1], x[0]
-		check[1 * n + 1] = false; //                x[1], x[1]
+		check[0 * n + 0] = false; // partial w.r.t. x[0], x[0]
+		check[0 * n + 1] = false; //                x[0], x[1]
+		check[0 * n + 2] = false; //                x[0], x[2]
+
+		check[1 * n + 0] = false; // partial w.r.t. x[1], x[0]
+		check[1 * n + 1] = true;  //                x[1], x[1]
+		check[1 * n + 2] = true;  //                x[1], x[2]
+
+		check[2 * n + 0] = false; // partial w.r.t. x[2], x[0]
+		check[2 * n + 1] = true;  //                x[2], x[1]
+		check[2 * n + 2] = false; //                x[2], x[2]
 	
 		// create function object F : X -> Y
 		CppAD::ADFun<double> F(X, Y);
@@ -954,7 +1274,7 @@ namespace {
 		result.swap(temp);
 	}
 
-	bool user_atomic_forward(
+	bool old_atomic_forward(
 		size_t                         id ,
 		size_t                          k , 
 		size_t                          n ,
@@ -980,7 +1300,7 @@ namespace {
 		return true; 
 	}
 
-	bool user_atomic_reverse(
+	bool old_atomic_reverse(
 		size_t                         id ,
 		size_t                          k , 
 		size_t                          n , 
@@ -991,7 +1311,7 @@ namespace {
 		const CppAD::vector<double>&   py )
 	{	return false; }
 
-	bool user_atomic_for_jac_sparse(
+	bool old_atomic_for_jac_sparse(
 		size_t                                  id ,
 		size_t                                   n ,
 		size_t                                   m ,
@@ -1000,7 +1320,7 @@ namespace {
 		CppAD::vector< std::set<size_t>  >&      s )
 	{	return false; }
 
-	bool user_atomic_rev_jac_sparse(
+	bool old_atomic_rev_jac_sparse(
 		size_t                                  id ,
 		size_t                                   n ,
 		size_t                                   m ,
@@ -1021,7 +1341,7 @@ namespace {
 		return true; 
 	}
 
-	bool user_atomic_rev_hes_sparse(
+	bool old_atomic_rev_hes_sparse(
 		size_t                                  id ,
 		size_t                                   n ,
 		size_t                                   m ,
@@ -1034,17 +1354,17 @@ namespace {
 	{	return false; }
 
 	CPPAD_USER_ATOMIC(
-		my_user_atomic             ,
+		my_old_atomic             ,
 		CppAD::vector              ,
 		double                     ,
-		user_atomic_forward        ,
-		user_atomic_reverse        ,
-		user_atomic_for_jac_sparse ,
-		user_atomic_rev_jac_sparse ,
-		user_atomic_rev_hes_sparse 
+		old_atomic_forward        ,
+		old_atomic_reverse        ,
+		old_atomic_for_jac_sparse ,
+		old_atomic_rev_jac_sparse ,
+		old_atomic_rev_hes_sparse 
 	)
 
-	bool user_atomic_test(void)
+	bool old_atomic_test(void)
 	{	bool ok = true;
 
 		using CppAD::AD;
@@ -1058,9 +1378,9 @@ namespace {
 
 		size_t id = 0;
 		// first call should stay in the tape
-		my_user_atomic(id++, ax, ay);
+		my_old_atomic(id++, ax, ay);
 		// second call will not get used
-		my_user_atomic(id++, ax, az);
+		my_old_atomic(id++, ax, az);
 		// create function
 		CppAD::ADFun<double> g(ax, ay);
 		// should have 1 + n + m + m varaibles
@@ -1108,12 +1428,225 @@ namespace {
 		// are identically equal
 		f.optimize();
 	
+		return ok;
+	}
+	// -----------------------------------------------------------------------
+	double floor(const double& x)
+	{	return std::floor(x); }
+	CPPAD_DISCRETE_FUNCTION(double, floor)
+	bool discrete_function(void)
+	{	bool ok = true;
+		using CppAD::vector;
+	
+		vector< CppAD::AD<double> > ax(1), ay(1);
+		ax[0] = 0.0; 
+		CppAD::Independent(ax);
+		ay[0] =  floor(ax[0]) + floor(ax[0]);  
+		CppAD::ADFun<double> f(ax, ay);
+	
+		size_t size_before = f.size_var();
+		f.optimize(); 
+		size_t size_after = f.size_var();
+		ok &= size_after + 1 == size_before;
+	
+		vector<double> x(1), y(1);
+		x[0] = -2.2;
+		y    = f.Forward(0, x);
+		ok &= y[0] == -6.0;
+	
+		return ok;
+	}
+	// ----------------------------------------------------------------
+	void i_algo( 
+		const CppAD::vector< CppAD::AD<double> >& ax ,
+		      CppAD::vector< CppAD::AD<double> >& ay )
+	{	ay[0] = 1.0 / ax[0]; }
+	//
+	// Test bug where atomic functions were not properly conditionally skipped.
+	bool cond_exp_skip_atomic(void)
+	{	bool ok = true;
+		using CppAD::AD;
+		using CppAD::vector;
+	
+		// Create a checkpoint version of the function i_algo
+		vector< AD<double> > au(1), av(1), aw(1);
+		au[0] = 1.0;
+		CppAD::checkpoint<double> i_check("i_check", i_algo, au, av);
+	
+		// independent variable vector 
+		vector< AD<double> > ax(2), ay(1);
+		ax[0] = 1.0;
+		ax[1] = 2.0;
+		Independent(ax);
+	
+		// call atomic function that does not get used
+		au[0] = ax[0];
+		i_check(au, av);
+		au[0] = ax[1];
+		i_check(au, aw);
+		AD<double> zero = 0.0;
+		ay[0] = CondExpGt(av[0], zero, av[0], aw[0]);
+
+		// create function object f : ax -> ay
+		CppAD::ADFun<double> f(ax, ay);
+
+		// run case that skips the second call to afun
+		// (can use trace in forward0sweep.hpp to see this).
+		vector<double> x(2), y_before(1), y_after(1);
+		x[0]      = 1.0;
+		x[1]      = 2.0;
+		y_before  = f.Forward(0, x);
+		f.optimize();
+		y_after   = f.Forward(0, x);
+	
+		ok &= y_before[0] == y_after[0];
+		
+		return ok;
+	}
+	//
+	// Test bug where conditional dependence did not pass through
+	// atomic functions
+	bool cond_exp_atomic_dependence(void)
+	{	bool ok = true;
+		using CppAD::AD;
+		using CppAD::vector;
+	
+		// Create a checkpoint version of the function i_algo
+		vector< AD<double> > au(1), av(1), aw(1);
+		au[0] = 1.0;
+		CppAD::checkpoint<double> i_check("i_check", i_algo, au, av);
+	
+		vector< AD<double> > ax(2), ay(1);
+		AD<double> zero = 0.0;  
+		ax[0] = 1.0;
+		ax[1] = 1.0;
+		Independent(ax);
+		av[0] = ax[0] + ax[1];
+		i_check(av, aw);
+		ay[0] = CondExpGt(aw[0], zero, zero, aw[0]);
+		CppAD::ADFun<double> f;
+		f.Dependent(ax, ay);
+
+		// run case that skips the second call to afun
+		// (but not for order zero)
+		vector<double> x(2), y_before(1), y_after(1);
+		vector<double> dx(2), dy_before(1), dy_after(1);
+		x[0]      = 1.0;
+		x[1]      = 1.0;
+		y_before  = f.Forward(0, x);
+		dx[0]     = 2.0;
+		dx[1]     = 2.0;
+		dy_before = f.Forward(1, dx);
+		f.optimize();
+		y_after   = f.Forward(0, x);
+		dy_after  = f.Forward(1, dx);
+
+		ok &= y_before[0]  == y_after[0];
+		ok &= dy_before[0] == dy_after[0];
+
+		return ok;
+	}
+	// -----------------------------------------------------------------------
+	// Test reverse mode conditionalay skipping commands.
+	template <class Type>
+	Type my_max(const CppAD::vector<Type>& arg)
+	{	Type res = arg[0];
+		for(size_t j = 0;j < arg.size(); j++)
+    		res = CondExpGt(res, arg[j], res, arg[j]);
+		return res;
+  	}
+	bool cond_exp_reverse(void)
+	{	bool ok = true;
+		size_t n = 3;
+		using CppAD::vector;
+		using CppAD::AD;
+	
+		vector< AD<double> > ax(n), ay(1);
+		for(size_t j = 0; j < n; j++)
+			ax[j] = 1.0;
+		Independent(ax);
+		ay[0] = my_max(ax) + my_max(ax);
+		CppAD::ADFun<double> f(ax, ay);
+	
+		f.optimize();
+	
+		vector<double> x(n), w(1), dx(n);
+		for(size_t j = 0;j < n; j++)
+			x[j] = double(j);
+		f.Forward(0, x);
+		w[0] = 1.0;
+		dx = f.Reverse(1, w);
+		for(size_t j = 0; j < n; j++)
+		{	if( j == n-1 )
+				ok &= dx[j] == 2.0;
+			else
+				ok &= dx[j] == 0.0;
+		}
+		return ok;
+	}
+	// Test case where an expression depends on both the true
+	// and false cases (bug fixed 2014-12-22)
+	bool cond_exp_both_true_and_false(void)
+	{	bool ok = true;
+		using CppAD::vector;
+		using CppAD::AD;
+
+		// f(x) = x[0] + x[0] if x[0] >= 3
+		//      = x[0] + x[1] otherwise 
+		vector< AD<double> > ax(2), ay(3);
+		ax[0] = 1.0;
+		ax[1] = 2.0;
+		Independent(ax);
+		AD<double> three(3);
+		AD<double> value = ax[0] + ax[1];
+		// a simple value
+		ay[0]  = CppAD::CondExpGe(ax[0], three, value, value);
+		// a  binary exprpression
+		ay[1]  = CppAD::CondExpGe(ax[0], three, ax[0]-ax[1], ax[0]-ax[1]);
+		// a unary expression
+		ay[2]  = CppAD::CondExpGe(ax[0], three, exp(ax[0]), exp(ax[0]) );
+		CppAD::ADFun<double> f(ax, ay);
+		f.optimize();
+
+		// check case where x[0] >= 3
+		vector<double> x(2), y(3);
+		x[0] = 4.0;
+		x[1] = 2.0;
+		y    = f.Forward(0, x);
+		ok  &= y[0] == x[0] + x[1];
+		ok  &= y[1] == x[0] - x[1];
+		ok  &= y[2] == exp(x[0]);
+
+		// check case where x[0] < 3
+		x[0] = 1.0;
+		x[1] = 2.0;
+		y    = f.Forward(0, x);
+		ok  &= y[0] == x[0] + x[1];
+		ok  &= y[1] == x[0] - x[1];
+		ok  &= y[2] == exp(x[0]);
+
 		return ok;
 	}
 }
 
 bool optimize(void)
 {	bool ok = true;
+
+	atomic_sparsity_option = CppAD::atomic_base<double>::bool_sparsity_enum;
+	for(size_t i = 0; i < 2; i++)
+	{	// check conditional expression sparsity pattern 
+		// (used to optimize calls to atomic functions). 
+		ok     &= atomic_cond_exp_sparsity();
+		// check optimizing out entire atomic function
+		ok     &= atomic_cond_exp();
+		// check optimizing out atomic arguments
+		ok     &= atomic_no_used();
+		ok     &= atomic_arguments();
+		atomic_sparsity_option = 
+			CppAD::atomic_base<double>::set_sparsity_enum;
+	}
+	// check nested conditional expressions
+	ok     &= nested_cond_exp();
 	// check reverse dependency analysis optimization
 	ok     &= depend_one();
 	ok     &= depend_two();
@@ -1127,16 +1660,27 @@ bool optimize(void)
 	ok     &= cummulative_sum();
 	ok     &= forward_csum();
 	ok     &= reverse_csum();
-	ok     &= forward_sparse_jacobian_csum();
-	ok     &= reverse_sparse_jacobian_csum();
-	ok     &= reverse_sparse_hessian_csum();
+	// sparsity patterns
+	ok     &= forward_sparse_jacobian();
+	ok     &= reverse_sparse_jacobian();
+	ok     &= reverse_sparse_hessian();
 	// check that CondExp properly detects dependencies
 	ok     &= cond_exp_depend();
-	// check user_atomic functions
-	ok     &= user_atomic_test();
+	// check old_atomic functions
+	ok     &= old_atomic_test();
 	// case where results are not identically equal
 	ok     &= not_identically_equal();
-
+	// case where a discrete function is used
+	ok     &= discrete_function();
+	// check conditional skip of an atomic function
+	ok     &= cond_exp_skip_atomic();
+	// check conditional dependence through atomic function
+	ok     &= cond_exp_atomic_dependence();
+	// check reverse mode conditional skipping
+	ok     &= cond_exp_reverse();
+	// check case where an expresion needed by both true and false case
+	ok     &=  cond_exp_both_true_and_false();
+	//
 	CppAD::user_atomic<double>::clear();
 	return ok;
 }
